@@ -1,18 +1,19 @@
-package Locale::CLDR v0.0.1;
+package Locale::CLDR v0.0.4;
 
 =encoding utf8
 
 =head1 NAME
 
-Locale::CLDR - Main Class for CLDR Locals
+Locale::CLDR - A Module to create locale objects with localisation data from the CLDR
 
 =head1 VERSION
 
-Version 0.0.1
+Version 0.0.4
 
 =head1 SYNOPSIS
 
-This module handles Locale Data from the CLDR.
+This module provides a locale object you can use to localise your output.
+The localisation data comes from the Unicode Common Locale Data Repository.
 
 =head1 USAGE
 
@@ -520,10 +521,6 @@ the data and you might want it
 All the above return a hashref keyed on date period
 with the value being the value for that date period
 
-The get_day_period() method will calculate the correct
-period for a given time and return the period name in
-the Locales language and script
-
 =item era_format_wide 
 
 =item era_format_abbreviated 
@@ -731,7 +728,7 @@ sub _build_break_rules {
 
 		foreach my $operand ($first, $second) {
 			if ($operand =~ m{ \S }msx) {
-				$operand = unicode_to_perl($operand);
+				$operand = _unicode_to_perl($operand);
 			}
 			else {
 				$operand = '.';
@@ -874,7 +871,7 @@ after 'BUILD' => sub {
 	my ($language_id, $script_id, $territory_id) = ($self->language_id, $self->script_id, $self->territory_id);
 	
 	unless ($language_id ne 'und' && $script_id && $territory_id ) {
-		$likely_subtag = $likely_subtags->{join '_', $language_id, $script_id, $territory_id};
+		$likely_subtag = $likely_subtags->{join '_', grep { length() } ($language_id, $script_id, $territory_id)};
 		
 		if (! $likely_subtag ) {
 			$likely_subtag = $likely_subtags->{join '_', $language_id, $territory_id};
@@ -894,7 +891,11 @@ after 'BUILD' => sub {
 	}
 		
 	if ($likely_subtag) {
-		$self->_set_likely_subtag(__PACKAGE__->new($likely_subtag));
+		my ($likely_language_id, $likely_script_id, $likely_territory_id) = split /_/, $likely_subtag;
+		$likely_language_id		= $language_id 	unless $language_id eq 'und';
+		$likely_script_id		= $script_id	if length $script_id;
+		$likely_territory_id	= $territory_id	if length $territory_id;
+		$self->_set_likely_subtag(__PACKAGE__->new(join '_',$likely_language_id, $likely_script_id, $likely_territory_id));
 	}
 };
 
@@ -1009,6 +1010,8 @@ sub _find_bundle {
 	my $id = $self->has_likely_subtag()
 		? $self->likely_subtag()->id()
 		: $self->id(); 
+		
+	
 	if ($self->method_cache->{$id}{$method_name}) {
 		return wantarray
 			? @{$self->method_cache->{$id}{$method_name}}
@@ -1309,23 +1312,28 @@ a locale key id as a string
 =cut
 
 sub key_name {
-	my ($self, $name) = @_;
+	my ($self, $key) = @_;
 
-	$name = lc $name;
+	$key = lc $key;
+	
 	my %key_aliases = $self->key_aliases;
 	my %key_names	= $self->key_names;
 	my %valid_keys	= $self->valid_keys;
 
-	my $alias = $key_aliases{$name} if exists $key_aliases{$name};
+	my $alias = $key_aliases{$key} // '';
+	my $name  = $key_names{$key} // '';
 
-	return '' unless exists $valid_keys{$name} || exists $valid_keys{$alias};
+	return '' unless exists $valid_keys{$key} || exists $valid_keys{$alias} || exists $valid_keys{$name};
 	my @bundles = $self->_find_bundle('display_name_key');
 	foreach my $bundle (@bundles) {
-		my $key = $bundle->display_name_key->{$name} // ( $alias ? $bundle->display_name_key->{$alias} : '');
-		return $key if length $key;
+		my $return = $bundle->display_name_key->{$key};
+		$return //= $bundle->display_name_key->{$alias}; 
+		$return //= $bundle->display_name_key->{$name}; 
+
+		return $return if defined $return && length $return;
 	}
 
-	return ucfirst ($key_names{$name} || $name);
+	return ucfirst ($key_names{$name} || $key_names{$alias} || $key_names{$key} || $key);
 }
 
 =item type_name($key, $type)
@@ -1345,14 +1353,15 @@ sub type_name {
 	my %valid_keys	= $self->valid_keys;
 	my %key_names	= $self->key_names;
 
-	my $alias = $key_aliases{$key} if exists $key_aliases{$key};
+	my $alias = $key_aliases{$key} // '';
+	my $name  = $key_names{$key}   // '';
 
-	return '' unless exists $valid_keys{$key} || $valid_keys{$alias};
-	return '' unless first { $_ eq $type } @{$valid_keys{$key} || $valid_keys{$alias} || []};
+	return '' unless exists $valid_keys{$key} || $valid_keys{$alias} || $valid_keys{$name};
+	return '' unless first { $_ eq $type } @{$valid_keys{$key} || []}, @{$valid_keys{$alias} || []}, @{$valid_keys{$name} || []};
 
 	my @bundles = $self->_find_bundle('display_name_type');
 	foreach my $bundle (@bundles) {
-		next unless my $types = $bundle->display_name_type->{$key_names{$key} || ($alias ? $key_names{$alias} : '')};
+		my $types = $bundle->display_name_type->{$key} // $bundle->display_name_type->{$alias} // $bundle->display_name_type->{$name};
 		my $type = $types->{$type};
 		return $type if defined $type;
 	}
@@ -1383,7 +1392,7 @@ sub measurement_system_name {
 	return '';
 }
 
-=item transform_name($measurement_system)
+=item transform_name($name)
 
 Returns the transform (transliteration) name in the current locales format. The transform must be
 a transform id as a string
@@ -1404,6 +1413,14 @@ sub transform_name {
 	return '';
 }
 
+=item code_pattern($type, $locale)
+
+This method formats a language, script or territory name, given as $type
+from $locale in a way expected by the current locale. If $locale is
+not passed in or is undef() the method uses the current locale.
+
+=cut
+
 sub code_pattern {
 	my ($self, $type, $locale) = @_;
 	$type = lc $type;
@@ -1926,7 +1943,7 @@ sub unit {
 	
 	# Check for a compound unit that we don't specifically have
 	if (! $format && (my ($dividend, $divisor) = $what =~ /^(.+)-per-(.+)$/)) {
-		return $self->unit_compound($number, $dividend, $divisor, $type);
+		return $self->_unit_compound($number, $dividend, $divisor, $type);
 	}
 	
 	$number = $self->format_number($number);
@@ -1935,7 +1952,7 @@ sub unit {
 	return $format =~ s/\{0\}/$number/gr;
 }
 
-sub unit_compound {
+sub _unit_compound {
 	my ($self, $number, $dividend_what, $divisor_what, $type) = @_;
 	
 	$type //= 'long';
@@ -1949,8 +1966,8 @@ sub unit_compound {
 	my @bundles = $self->_find_bundle('units');
 	my $format;
 	foreach my $bundle (@bundles) {
-		if (exists $bundle->units()->{$type}{per}{default}) {
-			$format = $bundle->units()->{$type}{per}{default};
+		if (exists $bundle->units()->{$type}{per}{''}) {
+			$format = $bundle->units()->{$type}{per}{''};
 			last;
 		}
 	}
@@ -1962,8 +1979,8 @@ sub unit_compound {
 		foreach my $alias (@aliases) {
 			$type = $alias->unit_alias()->{$original_type};
 			foreach my $bundle (@bundles) {
-				if (exists $bundle->units()->{$type}{per}{default}) {
-					$format = $bundle->units()->{$type}{per}{default};
+				if (exists $bundle->units()->{$type}{per}{''}) {
+					$format = $bundle->units()->{$type}{per}{''};
 					last;
 				}
 			}
@@ -2220,7 +2237,16 @@ sub list {
 	return $pattern;
 }
 
-# Stubs until I get onto numbers
+=item plural($number)
+
+This method takes a number and uses the locales pluralisation
+rules to calculate the type of pluralisation required for
+units, currencies and other data that changes depending on
+the plural state of the number
+
+=cut
+
+# Stub until I get onto numbers
 sub plural {
 	return 'one' if $_[1] =~ /1$/;
 	return 'other';
@@ -2458,6 +2484,14 @@ sub _build_quarter_stand_alone_narrow {
 	return $self->_build_any_quarter($type, $width);
 }
 
+=item get_day_period($time)
+
+This method will calculate the correct
+period for a given time and return the period name in
+the Locales language and script
+
+=cut
+
 sub get_day_period {
 	# Time in hhmm
 	my ($self, $time) = @_;
@@ -2480,6 +2514,7 @@ sub _build_any_am_pm {
 	my $default_calendar = $self->default_calendar();
 	my @result;
 	my @bundles = $self->_find_bundle('day_periods');
+	my %return;
 
 	BUNDLES: {
 		foreach my $bundle (@bundles) {
@@ -2502,11 +2537,13 @@ sub _build_any_am_pm {
 			
 			my $result = $am_pm->{$default_calendar}{$type}{$width};
 			
-			return $result if keys %$result;
+			foreach (keys %$result) {
+				$return{$_} = $result->{$_} unless exists $return{$_};
+			}
 		}
 	}
 
-	return {};
+	return \%return;
 }
 
 # The first 3 are to link in with Date::Time::Locale
@@ -2860,6 +2897,12 @@ sub _build_format_data {
 	return {};
 }
 
+=item format_for
+
+TODO fix this 
+
+=cut
+
 sub format_for {
 	my ($self, $format) = @_;
 
@@ -2938,7 +2981,7 @@ sub _build_prefers_24_hour_time {
 }
 
 # Sub to mangle Unicode regex to Perl regex
-sub unicode_to_perl {
+sub _unicode_to_perl {
 	my $regex = shift;
 
 	return '' unless length $regex;
@@ -2960,13 +3003,13 @@ sub unicode_to_perl {
 				)++                 # Do the inside set stuff one or more times without backtracking
 			\]						# End the set
 		)
-	/ convert($1) /xeg;
+	/ _convert($1) /xeg;
 	no warnings "experimental::regex_sets";
 	no warnings "deprecated"; # Because CLDR uses surrogates
 	return qr/$regex/x;
 }
 
-sub convert {
+sub _convert {
 	my $set = shift;
 	
 	# Some definitions
@@ -3040,15 +3083,14 @@ sub convert {
 	return "(?$set)";
 }
 
+=back
+
 =head1 AUTHOR
 
-John Imrie, C<< <j dot imrie1 at virginmedia.com> >>
+John Imrie, C<< <john dot imrie1 at gmail dot com> >>
 
 =head1 BUGS
 
-Please report any bugs or feature requests to me at the above email address 
-and ignore the CPAN stuff below for the present
-
 Please report any bugs or feature requests to C<bug-locale-cldr at rt.cpan.org>, or through
 the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Locale-CLDR>.  I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
